;Copyright (C) 2004-2007 John T. Haller

;Website: http://PortableApps.com/Notepad++Portable

;This software is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define NAME "Notepad++Portable"
!define FRIENDLYNAME "Notepad++ Portable"
!define APP "Notepad++"
!define VER "1.4.2.0"
!define WEBSITE "PortableApps.com/Notepad++Portable"
!define DEFAULTEXE "Notepad++.exe"
!define DEFAULTAPPDIR "Notepad++"
!define DEFAULTSETTINGSDIR "settings"

;=== Program Details
Name "${NAME}"
OutFile "${NAME}.exe"
Caption "${FRIENDLYNAME} | PortableApps.com"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${FRIENDLYNAME}"
VIAddVersionKey Comments "Allows ${APP} to be run from a removable drive.  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com (John T. Haller)"
VIAddVersionKey LegalCopyright "John T. Haller"
VIAddVersionKey FileDescription "${FRIENDLYNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${FRIENDLYNAME}"
VIAddVersionKey LegalTrademarks "PortableApps.com is a Trademark of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "${NAME}.exe"
;VIAddVersionKey PrivateBuild ""
;VIAddVersionKey SpecialBuild ""


;=== Runtime Switches
CRCCheck On
WindowIcon Off
SilentInstall Silent
AutoCloseWindow True
RequestExecutionLevel user

;=== Include
!include "GetParameters.nsh"
!include "Registry.nsh"
!include "FileFunc.nsh"
!include "ReplaceInFile.nsh"
!include "StrRep.nsh"
!insertmacro GetRoot

;=== Program Icon
Icon "${NAME}.ico"

Var PROGRAMDIRECTORY
Var SETTINGSDIRECTORY
Var ADDITIONALPARAMETERS
Var EXECSTRING
Var PROGRAMEXECUTABLE
Var INIPATH
Var DISABLESPLASHSCREEN
Var SECONDARYLAUNCH
Var DEFAULTLOCATION
Var USERTYPE


Section "Main"
	;=== Check if already running
	System::Call 'kernel32::CreateMutexA(i 0, i 0, t "${NAME}") i .r1 ?e'
	Pop $0
	StrCmp $0 0 CheckRunning
		StrCpy $SECONDARYLAUNCH "true"
		Goto CheckINI

	CheckRunning:
		FindProcDLL::FindProc "Notepad++.exe"
		StrCmp $R0 "1" "" CheckINI
			MessageBox MB_OK|MB_ICONINFORMATION `Another instance of ${APP} is already running.  Please close other instances of ${APP} before launching ${FRIENDLYNAME}.`
			Abort
		
	CheckINI:
		;=== Find the INI file, if there is one
		IfFileExists "$EXEDIR\${NAME}.ini" "" NoINI
			StrCpy $INIPATH "$EXEDIR"

		;=== Read the parameters from the INI file
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "${APP}Directory"
		StrCpy $PROGRAMDIRECTORY "$EXEDIR\$0"
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "SettingsDirectory"
		StrCpy $SETTINGSDIRECTORY "$EXEDIR\$0"

		;=== Check that the above required parameters are present
		IfErrors NoINI

		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "AdditionalParameters"
		StrCpy $ADDITIONALPARAMETERS $0
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "${APP}Executable"
		StrCpy $PROGRAMEXECUTABLE $0
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "DisableSplashScreen"
		StrCpy $DISABLESPLASHSCREEN $0

		;=== Any missing unrequired INI entries will be an empty string, ignore associated errors
		ClearErrors
		IfFileExists "$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" FoundProgramEXE NoProgramEXE

	NoINI:
		;=== No INI file, so we'll use the defaults
		StrCpy $ADDITIONALPARAMETERS ""
		StrCpy $PROGRAMEXECUTABLE "${DEFAULTEXE}"
		StrCpy $DISABLESPLASHSCREEN "false"
		StrCpy $DEFAULTLOCATION "true"

		IfFileExists "$EXEDIR\App\${DEFAULTAPPDIR}\${DEFAULTEXE}" "" CheckPortableProgramDIR
			StrCpy $PROGRAMDIRECTORY "$EXEDIR\App\${DEFAULTAPPDIR}"
			StrCpy $SETTINGSDIRECTORY "$EXEDIR\Data\${DEFAULTSETTINGSDIR}"
			GoTo FoundProgramEXE

		CheckPortableProgramDIR:
			IfFileExists "$EXEDIR\${NAME}\App\${DEFAULTAPPDIR}\${DEFAULTEXE}" "" NoProgramEXE
			StrCpy $PROGRAMDIRECTORY "$EXEDIR\${NAME}\App\${DEFAULTAPPDIR}"
			StrCpy $SETTINGSDIRECTORY "$EXEDIR\${NAME}\Data\${DEFAULTSETTINGSDIR}"
			GoTo FoundProgramEXE

	NoProgramEXE:
		;=== Program executable not where expected
		MessageBox MB_OK|MB_ICONEXCLAMATION `$PROGRAMEXECUTABLE was not found.  Please check your configuration`
		Abort
		
	FoundProgramEXE:
		StrCmp $SECONDARYLAUNCH "true" GetPassedParameters
		StrCmp $DISABLESPLASHSCREEN "true" SkipSplashScreen
			;=== Show the splash screen before processing the files
			InitPluginsDir
			File /oname=$PLUGINSDIR\splash.jpg "${NAME}.jpg"
			newadvsplash::show /NOUNLOAD 1200 100 0 -1 /L "$PLUGINSDIR\splash.jpg"

	SkipSplashScreen:
		;=== Check for data files
		IfFileExists "$PROGRAMDIRECTORY\config.xml" GetPassedParameters ;=== settings already in program directory
		IfFileExists "$SETTINGSDIRECTORY\config.xml" MoveSettings ;=== settings found in data directory
		
		;=== Copy the default settings files
		StrCmp $DEFAULTLOCATION "true" "" GetPassedParameters ;=== if not default location, user is on their own
		CreateDirectory "$SETTINGSDIRECTORY"
		CopyFiles /SILENT "$EXEDIR\App\DefaultData\settings\*.*" "$SETTINGSDIRECTORY\"
		CreateDirectory "$SETTINGSDIRECTORY\plugins"
		CopyFiles /SILENT "$EXEDIR\App\DefaultData\settings\plugins\*.*" "$SETTINGSDIRECTORY\plugins\"
		CreateDirectory "$SETTINGSDIRECTORY\plugins\config"
	
	MoveSettings:
		Rename "$SETTINGSDIRECTORY\config.xml" "$PROGRAMDIRECTORY\config.xml"
		Rename "$SETTINGSDIRECTORY\contextMenu.xml" "$PROGRAMDIRECTORY\contextMenu.xml"
		Rename "$SETTINGSDIRECTORY\ConvertExt.enc" "$PROGRAMDIRECTORY\ConvertExt.enc"
		Rename "$SETTINGSDIRECTORY\ConvertExt.ini" "$PROGRAMDIRECTORY\ConvertExt.ini"
		Rename "$SETTINGSDIRECTORY\ConvertExt.lng" "$PROGRAMDIRECTORY\ConvertExt.lng"
		Rename "$SETTINGSDIRECTORY\insertExt.ini" "$PROGRAMDIRECTORY\insertExt.ini"
		Rename "$SETTINGSDIRECTORY\NPPTextFX.ini" "$PROGRAMDIRECTORY\NPPTextFX.ini"
		Rename "$SETTINGSDIRECTORY\session.xml" "$PROGRAMDIRECTORY\session.xml"
		Rename "$SETTINGSDIRECTORY\shortcuts.xml" "$PROGRAMDIRECTORY\shortcuts.xml"
		Rename "$SETTINGSDIRECTORY\SpellChecker.ini" "$PROGRAMDIRECTORY\SpellChecker.ini"
		Rename "$SETTINGSDIRECTORY\stylers.xml" "$PROGRAMDIRECTORY\stylers.xml"
		Rename "$SETTINGSDIRECTORY\nativeLang.xml" "$PROGRAMDIRECTORY\nativeLang.xml"
		Rename "$SETTINGSDIRECTORY\plugins\NPPTextFX.ini" "$PROGRAMDIRECTORY\plugins\NPPTextFX.ini"
		Rename "$SETTINGSDIRECTORY\plugins\config\Explorer.ini" "$PROGRAMDIRECTORY\plugins\config\Explorer.ini"
		Rename "$SETTINGSDIRECTORY\plugins\config\NppExec.ini" "$PROGRAMDIRECTORY\plugins\config\NppExec.ini"
		Rename "$SETTINGSDIRECTORY\plugins\config\SpellChecker.ini" "$PROGRAMDIRECTORY\plugins\config\SpellChecker.ini"
		Rename "$SETTINGSDIRECTORY\nativeLang.xml" "$PROGRAMDIRECTORY\nativeLang.xml"
		
		;=== Check for registry permissions and backup the key
		UserInfo::GetAccountType
		Pop $0
		StrCpy $USERTYPE $0
		StrCmp $USERTYPE "Guest" UpdateLocations
		StrCmp $USERTYPE "User" UpdateLocations
		${registry::KeyExists} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Notepad++_file" $R0
		StrCmp $R0 "-1" UpdateLocations
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Notepad++_file" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Notepad++_file-BackupByNotepad++Portable" $R0
	
	UpdateLocations:
		;=== Update files for new location
		${GetRoot} $EXEDIR $0
		ReadINIStr $1 "$SETTINGSDIRECTORY\Notepad++PortableSettings.ini" "Notepad++PortableSettings" "LastDriveLetter"
		StrCmp $1 "" StoreCurrentDriveLetter
		StrCmp $1 $0 StoreCurrentDriveLetter
		IfFileExists "$PROGRAMDIRECTORY\session.xml" "" UpdateExplorerPlugin
		${ReplaceInFile} "$PROGRAMDIRECTORY\session.xml" "$1\" "$0\"
		Delete "$PROGRAMDIRECTORY\session.xml.old"
	
	UpdateExplorerPlugin:
		IfFileExists "$PROGRAMDIRECTORY\plugins\config\Explorer.ini" "" UpdateConfig
			${ReplaceInFile} "$PROGRAMDIRECTORY\plugins\config\Explorer.ini" "$1\" "$0\"
			Delete "$PROGRAMDIRECTORY\plugins\config\Explorer.ini.old"

	UpdateConfig:
		IfFileExists "$PROGRAMDIRECTORY\config.xml" "" StoreCurrentDriveLetter
			${ReplaceInFile} "$PROGRAMDIRECTORY\config.xml" "$1\" "$0\"
			Delete "$PROGRAMDIRECTORY\config.xml.old"
		
	StoreCurrentDriveLetter:
		WriteINIStr "$SETTINGSDIRECTORY\Notepad++PortableSettings.ini" "Notepad++PortableSettings" "LastDriveLetter" "$0"

	GetPassedParameters:
		;=== Get any passed parameters
		Call GetParameters
		Pop $0
		StrCmp "'$0'" "''" "" LaunchProgramParameters

		;=== No parameters
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE"`
		Goto AdditionalParameters

	LaunchProgramParameters:
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" $0`

	AdditionalParameters:
		StrCmp $ADDITIONALPARAMETERS "" LaunchNow

		;=== Additional Parameters
		StrCpy $EXECSTRING `$EXECSTRING $ADDITIONALPARAMETERS`

	LaunchNow:
		StrCmp $SECONDARYLAUNCH "true" LaunchAndExit
		ExecWait $EXECSTRING

		;=== Move settings back
		Delete "$SETTINGSDIRECTORY\config.xml"
		Delete "$SETTINGSDIRECTORY\contextMenu.xml"
		Delete "$SETTINGSDIRECTORY\ConvertExt.enc"
		Delete "$SETTINGSDIRECTORY\ConvertExt.ini"
		Delete "$SETTINGSDIRECTORY\ConvertExt.lng"
		Delete "$SETTINGSDIRECTORY\insertExt.ini"
		Delete "$SETTINGSDIRECTORY\NPPTextFX.ini"
		Delete "$SETTINGSDIRECTORY\session.xml"
		Delete "$SETTINGSDIRECTORY\shortcuts.xml"
		Delete "$SETTINGSDIRECTORY\SpellChecker.ini"
		Delete "$SETTINGSDIRECTORY\stylers.xml"
		Delete "$SETTINGSDIRECTORY\nativeLang.xml"
		Delete "$SETTINGSDIRECTORY\plugins\NPPTextFX.ini"
		Delete "$SETTINGSDIRECTORY\plugins\config\Explorer.ini"
		Delete "$SETTINGSDIRECTORY\plugins\config\NppExec.ini"
		Delete "$SETTINGSDIRECTORY\plugins\config\SpellChecker.ini"
		Rename "$PROGRAMDIRECTORY\config.xml" "$SETTINGSDIRECTORY\config.xml"
		Rename "$PROGRAMDIRECTORY\contextMenu.xml" "$SETTINGSDIRECTORY\contextMenu.xml"
		Rename "$PROGRAMDIRECTORY\ConvertExt.enc" "$SETTINGSDIRECTORY\ConvertExt.enc"
		Rename "$PROGRAMDIRECTORY\ConvertExt.ini" "$SETTINGSDIRECTORY\ConvertExt.ini"
		Rename "$PROGRAMDIRECTORY\ConvertExt.lng" "$SETTINGSDIRECTORY\ConvertExt.lng"
		Rename "$PROGRAMDIRECTORY\insertExt.ini" "$SETTINGSDIRECTORY\insertExt.ini"
		Rename "$PROGRAMDIRECTORY\NPPTextFX.ini" "$SETTINGSDIRECTORY\NPPTextFX.ini"
		Rename "$PROGRAMDIRECTORY\session.xml" "$SETTINGSDIRECTORY\session.xml"
		Rename "$PROGRAMDIRECTORY\shortcuts.xml" "$SETTINGSDIRECTORY\shortcuts.xml"
		Rename "$PROGRAMDIRECTORY\SpellChecker.ini" "$SETTINGSDIRECTORY\SpellChecker.ini"
		Rename "$PROGRAMDIRECTORY\stylers.xml" "$SETTINGSDIRECTORY\stylers.xml"
		Rename "$PROGRAMDIRECTORY\nativeLang.xml" "$SETTINGSDIRECTORY\nativeLang.xml"
		Rename "$PROGRAMDIRECTORY\plugins\NPPTextFX.ini" "$SETTINGSDIRECTORY\plugins\NPPTextFX.ini"
		Rename "$PROGRAMDIRECTORY\plugins\config\Explorer.ini" "$SETTINGSDIRECTORY\plugins\config\Explorer.ini"
		Rename "$PROGRAMDIRECTORY\plugins\config\NppExec.ini" "$SETTINGSDIRECTORY\plugins\config\NppExec.ini"
		Rename "$PROGRAMDIRECTORY\plugins\config\SpellChecker.ini" "$SETTINGSDIRECTORY\plugins\config\SpellChecker.ini"
		
		;=== Restore the registry key
		StrCmp $USERTYPE "Guest" TheEnd
		StrCmp $USERTYPE "User" TheEnd
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Notepad++_file" $R0
		${registry::KeyExists} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Notepad++_file-BackupByNotepad++Portable" $R0
		StrCmp $R0 "-1" TheEnd
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Notepad++_file-BackupByNotepad++Portable" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Notepad++_file" $R0
		Goto TheEnd
	
	LaunchAndExit:
		Exec $EXECSTRING

	TheEnd:
		${registry::Unload}
		newadvsplash::wait
SectionEnd